

call sysproc.set_routine_opts('ISOLATION UR') @
drop specific procedure create_part_gtt@
CREATE PROCEDURE CREATE_PARTITION_TEMP_TABLE
                                   ( IN i_tabname   VARCHAR(50),
                                     IN i_schema    VARCHAR(30), 
                                     IN i_preview_only_ind   CHAR(1),
                                     IN i_locktimeout_cnt INTEGER,
                                     OUT o_temp_table  VARCHAR(50),     -- the name of the _gtt
                                     OUT o_sqlcode  INT  )
SPECIFIC CREATE_PART_GTT
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  CREATE TEMP TABLE
--
--
-- Description:
--     Given the name of a partioned table find the oldest partition, 
--     create a global temp table, and copy all eligible rows from the oldest partition
--     into the global temp table.
-- 
--
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
--    The insert statement used to populate the global temporary table changes 
--    between R1 and R2 bear that in mind.  Both versions are resident, but 
--    one is commented out.
--
--    Also this proc expects a user 32k temp tablespace to exist and be accessible.
--    
--           
-- Usage:
--   call create_partition_temp_table ('FCM_AUTH', 'FCMCORE', '0', ?, ?) ;
--
--
-- History:    
--      2007/11/22  created       
--      2009/06/18  Updated for V3. Looks at xqo_cust_num = grx_cust_num, instead of acct_num       
--      2009/09/03  leushe  Added support for the LOCKTIMEOUT_RETRY_COUNT (i_locktimeout_cnt)
--                          to pass to exec_sql_and_log procedure
--      2010/10/22  leushe  Changed references to GRX_, GRA_ and GRC_ to XQO_,AQO_, and blank respectively--           
--      2010/12/22  leushe  fixed ambigous reference to xqo_Cust_num
--      2011/01/27  leushe  changed the preservation of CF txn logic to determine which "alert type" fields
--                          are on the txn table, and compare those columns with the alert_value of the alert
--                          by alert_type    
--      2012/09/11  leushe  changed the preservation of CF txn logic to determine which "alert type" fields
--                          to use an OR clause        
--      2015/04/15  leushe  change v_dynStmt to varchar 7000. Modified c_prep cursor to join to new table FCM_ALERT_TYPE_CALL_RESULT
--                          and removed the xqo_cust_num elimination from the where clause. This new table contains the Result_id, 
--                          per alert type to preserve in CF
--                          The INSERT statement dynamically generated modified to use v_result_id       
--      2015/05/21  leushe  In db2 9.5, the || function does not implicitly convert an integer to string. Added the CHAR function 
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE v_table_part_name VARCHAR(50);
  DECLARE v_partition       VARCHAR(50);
  DECLARE v_dynStmt         VARCHAR(7000);
  DECLARE v_lowerbound      VARCHAR(36);
  DECLARE v_upperbound      VARCHAR(36);
  DECLARE v_user_tablespace VARCHAR(64);
  DECLARE v_temp_table      VARCHAR(50);
  DECLARE l_counter INTEGER DEFAULT 0; 
  DECLARE at_end SMALLINT DEFAULT 0;    
  DECLARE v_fieldname VARCHAR(50);
  DECLARE v_alert_type CHAR(1); 
  DECLARE v_result_id INTEGER; 
  
  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;
  DECLARE not_found CONDITION FOR SQLSTATE '02000';
  
  DECLARE v_stmt STATEMENT;

--' AND ( alrt.alert_Value = txn.'||rtrim(field_name)||'  and alrt.alert_type = '''||alert_Type||''')' as xyz
    DECLARE c_prep CURSOR FOR
	  select distinct field_name,att.alert_Type,att.result_id
            from fcm_alert_Type at,FCM_ALERT_TYPE_CALL_RESULT att, fcm_field_definition fd, fcm_transaction_Definition td,
	    FCM_transaction_component tc, fcm_component_segment cs,fcm_segment_Definition sd,fcm_component_value cv
	  where fd.field_id = at.KEY_FIELD_ID
          and at.alert_type = att.alert_Type
          and sd.segment_id = fd.SEGMENT_ID
          and cs.SEGMENT_ID = sd.SEGMENT_ID
          and tc.TRANSACTION_ID = td.TRANSACTION_ID
          and tc.COMPONENT_VALUE_ID = cv.COMPONENT_VALUE_ID
          and cs.COMPONENT_VALUE_ID = cv.COMPONENT_VALUE_ID
          and td.SOR_TABLE_NAME = i_tabname
          and att.logical_del_ind='0';
--          and fd.field_name <> 'xqo_cust_num';   
   
  DECLARE CONTINUE HANDLER for not_found
      SET at_end = 1; 

  --
  -- get the minimum lowerbound value of the first (oldest)  non CF partition 
  --
  SET v_lowerbound =
        ( select cast( min(lowvalue) as varchar(36) )
          from syscat.datapartitions b
          where b.tabname = i_tabname
          and b.tabschema = i_schema 
          and b.datapartitionname not like '%CF%'
          and b.datapartitionname not like '%THE_END%' 
          and status != 'I' )
        ;

  --
  -- get the minimum upperbound value of the first (oldest) non CF partition 
  --
  SET v_upperbound =
        ( select cast( min(highvalue) as varchar(36) )
          from syscat.datapartitions b
          where b.tabname = i_tabname
          and b.tabschema = i_schema 
          and b.datapartitionname not like '%CF%'
          and b.datapartitionname not like '%THE_END%' 
          and status != 'I' )
        ;

  --
  -- get the name of a user temporary tablespace with a block size of 32k 
  -- this tablespace will house the global temporary table
  --
  SET v_user_tablespace =  
        ( select cast ( min(tbspace) as varchar(64) )
          from syscat.tablespaces t
          where t.datatype = 'U'
          and t.pagesize = 32768 )
      ;


  --
  -- create the name of the global temporary table
  --
  SET v_temp_table = i_tabname || '_GTT' ;


  --
  -- create the dynamic sql to create the gtt
  -- 
  SET v_dynStmt = 'DECLARE GLOBAL TEMPORARY TABLE ' || v_temp_table
                  || ' LIKE '  || i_schema || '.'  || i_tabname
                  || ' ON COMMIT PRESERVE ROWS NOT LOGGED IN ' 
                  || v_user_tablespace 
                ;

  call exec_sql_and_log(i_tabname, i_schema, v_partition, i_preview_only_ind, i_locktimeout_cnt, v_dynStmt, o_sqlcode );

  IF ( o_sqlcode != 0 ) THEN
     RETURN ;
  END IF;


  SET v_dynStmt = 'INSERT INTO SESSION' || '.' || v_temp_table
                  || ' SELECT * FROM ' || i_schema || '.' || i_tabname
                  || ' as txn WHERE EXISTS '
                 || '( SELECT 1 FROM ' || i_schema || '.' || 'fcm_alert as alrt WHERE  ' ;                  
 
   	open c_prep ;


    while ( at_end = 0 ) do
      FETCH c_prep into v_fieldname, v_alert_type,v_result_id;
      	IF ( at_end=0 ) THEN
      	  IF (l_counter=0) THEN
      	    	set v_dynStmt = v_dynStmt||'  ( (coalesce(result_id,0) = '||char(v_result_id)||' and  alrt.alert_Value = txn.'||rtrim(v_fieldname)||'  and alrt.alert_type = '''||v_alert_type||''')';
          ELSE
            set v_dynStmt = v_dynStmt||' OR ( coalesce(result_id,0)= '||char(v_result_id)||' and alrt.alert_Value = txn.'||rtrim(v_fieldname)||'  and alrt.alert_type = '''||v_alert_type||''')';
          END IF;
        

      	END IF; 
      set l_counter=l_counter+1;
    end while;
    
    -- BAD makes assumption that the l_counter was > 0 after while loop
    set v_dynStmt = v_dynStmt||') ) AND rqo_proc_utc_datetime BETWEEN ' || v_lowerbound 
                  || ' AND ' || v_upperbound 
                ;

  call exec_sql_and_log(i_tabname, i_schema, v_partition, i_preview_only_ind, i_locktimeout_cnt, v_dynStmt, o_sqlcode );

  IF ( o_sqlcode != 0 ) THEN
     RETURN ;
  END IF;


  SET o_temp_table = v_temp_table;

END P1
@

