
drop specific procedure exec_sql_and_log @
drop specific procedure exec_sqlparm_and_log @
call sysproc.set_routine_opts('ISOLATION UR') @


CREATE PROCEDURE EXEC_SQL_AND_LOG ( IN i_tabname          VARCHAR(50),
                                    IN i_schema           VARCHAR(30),
                                    IN i_partition        VARCHAR(50),
                                    IN i_preview_only_ind CHAR(1),
                                    IN i_locktimeout_cnt  INTEGER,
                                    IN i_dynstmt          VARCHAR(7000),
                                    OUT o_sqlcode         INTEGER)


SPECIFIC exec_sql_and_log
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  
--
--
-- Description:
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
-- 
--
-- Usage:
--
--
-- History:    
--      2007/08/31   jicimi   created       
--      2007/09/20   jicimi   added get_diagnostic call      
--      2007/10/31   jicimi   added loop to trap -911s and 
--                            retry upto 3 times
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT (i_locktimeout_cnt)
--                            the retry count is not passed in, so its configurable
--      2011/11/30   leushe   Added support to retry for sqlstates 55057 and 20285 for db2 9.7 Linux AIC contention
--      2012/04/26   leushe   instead of looping for 30secs, I added the call to the Sleep Proc
--      2015/04/15   leushe   change i_dynstmt to varchar 7000 and v_sqlmsg to varchar 23000      
--           
--          
--         
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;
  DECLARE v_sqlcode integer DEFAULT 0 ;
  DECLARE v_sqlstate char(5) DEFAULT '00000';
  DECLARE v_sqlmsg VARCHAR(23000) DEFAULT '';  
  DECLARE v_rows integer DEFAULT 0 ;
  DECLARE v_attempts integer DEFAULT 1 ;
  DECLARE v_stmt STATEMENT;


  WHILE  ( v_attempts <= i_locktimeout_cnt ) DO

      SET v_sqlstate = '00000';
      SET v_sqlcode = 0;
      SET v_sqlmsg = '';
      IF ( i_preview_only_ind = '0' ) THEN
        ANONYMOUS_BLOCK: BEGIN
        DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
        BEGIN
           get diagnostics exception 1 v_sqlmsg = message_text;
           --select sqlcode, sqlstate,substr(SYSPROC.SQLERRM(coalesce(SQLCODE,0)),1,30000) into v_sqlcode, v_sqlstate,v_sqlmsg from sysibm.sysdummy1 ;
            select sqlcode, sqlstate into v_sqlcode, v_sqlstate from sysibm.sysdummy1 ;
         END;
        --SIGNAL SQLSTATE '99001';
        PREPARE v_stmt FROM i_dynStmt;
        EXECUTE IMMEDIATE i_dynStmt;
        GET DIAGNOSTICS v_rows = ROW_COUNT;
        END ANONYMOUS_BLOCK;
      END IF;
    
    
      CALL audit_partition_action  (i_tabname, i_schema, i_partition, 
                        i_dynStmt, i_preview_only_ind, v_rows,  v_sqlcode, v_sqlstate,v_sqlmsg );
      --40001 is deadlock/timeout
      IF ( (v_sqlstate != '40001') AND
           (v_sqlstate != '55057') AND 
           (posstr(v_sqlmsg,'20285') = 0) ) THEN

          SET v_attempts = i_locktimeout_cnt+1 ;

        ELSE
          SET v_attempts = v_attempts + 1 ;         
		  call sleep(30);
      END IF;

  END WHILE;

  SET o_sqlcode = v_sqlcode ;

END P1
@



CREATE PROCEDURE EXEC_SQL_AND_LOG ( IN i_tabname          VARCHAR(50),
                                    IN i_schema           VARCHAR(30),
                                    IN i_partition        VARCHAR(50),
                                    IN i_preview_only_ind CHAR(1),
                                    IN i_locktimeout_cnt  INTEGER,                                    
                                    IN i_dynstmt          VARCHAR(7000),
                                    IN i_parameter        VARCHAR(1024),
                                    OUT o_sqlcode         INTEGER )

SPECIFIC exec_sqlparm_and_log
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  
--
--
-- Description:
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
-- 
--
-- Usage:
--
--
-- History:    
--      2007/09/06   jicimi   created       
--      2007/10/31   jicimi   added loop to trap -911s and 
--                            retry upto 3 time
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT (i_locktimeout_cnt)
--                            the retry count is not passed in, so its configurable 
--      2011/11/30   leushe   Added support to retry for sqlstates 55057 and 20285 for db2 9.7 Linux AIC contention
--      2012/04/26   leushe   instead of looping for 30secs, I added the call to the Sleep Proc     
--      2015/04/15   leushe   change i_dynstmt to varchar 7000 and v_sqlmsg to varchar 23000      
--           
--          
--         
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;
  DECLARE v_sqlcode integer DEFAULT 0 ;
  DECLARE v_sqlstate char(5) DEFAULT '00000';
  DECLARE v_sqlmsg VARCHAR(23000) DEFAULT '';
  DECLARE v_rows integer DEFAULT 0 ;
  DECLARE v_attempts integer DEFAULT 1 ;
  DECLARE v_stmt STATEMENT;
  DECLARE s1 STATEMENT;


  WHILE  ( v_attempts <= i_locktimeout_cnt ) DO

      SET v_sqlstate = '00000';
      SET v_sqlcode = 0 ;
      SET v_sqlmsg = '';
      
      IF ( i_preview_only_ind = '0' ) THEN
    
        ANONYMOUS_BLOCK: BEGIN
        DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
        BEGIN
           get diagnostics exception 1 v_sqlmsg = message_text;
           select sqlcode, sqlstate into v_sqlcode, v_sqlstate from sysibm.sysdummy1 ;
         END;
        PREPARE v_stmt FROM i_dynStmt;
        EXECUTE v_stmt USING i_parameter ;
        GET DIAGNOSTICS v_rows = ROW_COUNT;
    
        END ANONYMOUS_BLOCK;
      END IF;
    

      CALL audit_partition_action  (i_tabname, i_schema, i_partition, 
                        i_dynStmt||' ?='||i_parameter, i_preview_only_ind, v_rows,
                        v_sqlcode, v_sqlstate,v_sqlmsg );

      IF ( (v_sqlstate != '40001') AND
           (v_sqlstate != '55057') AND 
           (posstr(v_sqlmsg,'20285') = 0) ) THEN
          SET v_attempts = i_locktimeout_cnt+1 ;
      ELSE
          SET v_attempts = v_attempts + 1 ;
          call sleep(30);
      END IF;

  END WHILE;

  SET o_sqlcode = v_sqlcode ;


END P1
@
