call sysproc.set_routine_opts('ISOLATION UR') @

drop specific procedure update_cf_partition @
drop specific procedure exec_sql_and_log @
drop specific procedure exec_sqlparm_and_log @
drop specific procedure roll_partition    @

CREATE PROCEDURE EXEC_SQL_AND_LOG ( IN i_tabname          VARCHAR(50),
                                    IN i_schema           VARCHAR(30),
                                    IN i_partition        VARCHAR(50),
                                    IN i_preview_only_ind CHAR(1),
                                    IN i_locktimeout_cnt  INTEGER,
                                    IN i_dynstmt          VARCHAR(1024),
                                    OUT o_sqlcode         INTEGER)


SPECIFIC exec_sql_and_log
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  
--
--
-- Description:
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
-- 
--
-- Usage:
--
--
-- History:    
--      2007/08/31   jicimi   created       
--      2007/09/20   jicimi   added get_diagnostic call      
--      2007/10/31   jicimi   added loop to trap -911s and 
--                            retry upto 3 times
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT (i_locktimeout_cnt)
--                            the retry count is not passed in, so its configurable
--      2011/11/30   leushe   Added support to retry for sqlstates 55057 and 20285 for db2 9.7 Linux AIC contention
--      2012/04/26   leushe   instead of looping for 30secs, I added the call to the Sleep Proc
--           
--          
--         
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;
  DECLARE v_sqlcode integer DEFAULT 0 ;
  DECLARE v_sqlstate char(5) DEFAULT '00000';
  DECLARE v_sqlmsg VARCHAR(30000) DEFAULT '';  
  DECLARE v_rows integer DEFAULT 0 ;
  DECLARE v_attempts integer DEFAULT 1 ;
  DECLARE v_stmt STATEMENT;


  WHILE  ( v_attempts <= i_locktimeout_cnt ) DO

      SET v_sqlstate = '00000';
      SET v_sqlcode = 0;
      SET v_sqlmsg = '';
      IF ( i_preview_only_ind = '0' ) THEN
        ANONYMOUS_BLOCK: BEGIN
        DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
        BEGIN
           get diagnostics exception 1 v_sqlmsg = message_text;
           --select sqlcode, sqlstate,substr(SYSPROC.SQLERRM(coalesce(SQLCODE,0)),1,30000) into v_sqlcode, v_sqlstate,v_sqlmsg from sysibm.sysdummy1 ;
            select sqlcode, sqlstate into v_sqlcode, v_sqlstate from sysibm.sysdummy1 ;
         END;
        --SIGNAL SQLSTATE '99001';
        PREPARE v_stmt FROM i_dynStmt;
        EXECUTE IMMEDIATE i_dynStmt;
        GET DIAGNOSTICS v_rows = ROW_COUNT;
        END ANONYMOUS_BLOCK;
      END IF;
    
    
      CALL audit_partition_action  (i_tabname, i_schema, i_partition, 
                        i_dynStmt, i_preview_only_ind, v_rows,  v_sqlcode, v_sqlstate,v_sqlmsg );
      --40001 is deadlock/timeout
      IF ( (v_sqlstate != '40001') AND
           (v_sqlstate != '55057') AND 
           (posstr(v_sqlmsg,'20285') = 0) ) THEN

          SET v_attempts = i_locktimeout_cnt+1 ;

        ELSE
          SET v_attempts = v_attempts + 1 ;         
		  call sleep(30);
      END IF;

  END WHILE;

  SET o_sqlcode = v_sqlcode ;

END P1
@



CREATE PROCEDURE EXEC_SQL_AND_LOG ( IN i_tabname          VARCHAR(50),
                                    IN i_schema           VARCHAR(30),
                                    IN i_partition        VARCHAR(50),
                                    IN i_preview_only_ind CHAR(1),
                                    IN i_locktimeout_cnt  INTEGER,                                    
                                    IN i_dynstmt          VARCHAR(1024),
                                    IN i_parameter        VARCHAR(1024),
                                    OUT o_sqlcode         INTEGER )

SPECIFIC exec_sqlparm_and_log
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  
--
--
-- Description:
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
-- 
--
-- Usage:
--
--
-- History:    
--      2007/09/06   jicimi   created       
--      2007/10/31   jicimi   added loop to trap -911s and 
--                            retry upto 3 time
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT (i_locktimeout_cnt)
--                            the retry count is not passed in, so its configurable 
--      2011/11/30   leushe   Added support to retry for sqlstates 55057 and 20285 for db2 9.7 Linux AIC contention
--      2012/04/26   leushe   instead of looping for 30secs, I added the call to the Sleep Proc     
--           
--          
--         
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;
  DECLARE v_sqlcode integer DEFAULT 0 ;
  DECLARE v_sqlstate char(5) DEFAULT '00000';
  DECLARE v_sqlmsg VARCHAR(30000) DEFAULT '';
  DECLARE v_rows integer DEFAULT 0 ;
  DECLARE v_attempts integer DEFAULT 1 ;
  DECLARE v_stmt STATEMENT;
  DECLARE s1 STATEMENT;


  WHILE  ( v_attempts <= i_locktimeout_cnt ) DO

      SET v_sqlstate = '00000';
      SET v_sqlcode = 0 ;
      SET v_sqlmsg = '';
      
      IF ( i_preview_only_ind = '0' ) THEN
    
        ANONYMOUS_BLOCK: BEGIN
        DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
        BEGIN
           get diagnostics exception 1 v_sqlmsg = message_text;
           select sqlcode, sqlstate into v_sqlcode, v_sqlstate from sysibm.sysdummy1 ;
         END;
        PREPARE v_stmt FROM i_dynStmt;
        EXECUTE v_stmt USING i_parameter ;
        GET DIAGNOSTICS v_rows = ROW_COUNT;
    
        END ANONYMOUS_BLOCK;
      END IF;
    

      CALL audit_partition_action  (i_tabname, i_schema, i_partition, 
                        i_dynStmt||' ?='||i_parameter, i_preview_only_ind, v_rows,
                        v_sqlcode, v_sqlstate,v_sqlmsg );

      IF ( (v_sqlstate != '40001') AND
           (v_sqlstate != '55057') AND 
           (posstr(v_sqlmsg,'20285') = 0) ) THEN
          SET v_attempts = i_locktimeout_cnt+1 ;
      ELSE
          SET v_attempts = v_attempts + 1 ;
          call sleep(30);
      END IF;

  END WHILE;

  SET o_sqlcode = v_sqlcode ;


END P1
@





CREATE PROCEDURE UPDATE_CF_PARTITION ( IN i_tabname   VARCHAR(50),
                                       IN i_schema    VARCHAR(30), 
                                       IN i_cf_partition VARCHAR(50),        -- the cf partition
                                       IN i_preview_only_ind   CHAR(1),
                                       IN i_locktimeout_cnt   INTEGER,
                                       IN i_partition_datatype CHAR(9),
                                       IN i_detach_wait_sec INTEGER,
                                       OUT o_sqlcode  INT  )
SPECIFIC UPDATE_CF_PARTITION
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  UPDATE_CF_PARTITION
--
--
-- Description:  
--     This should be the normal code path.   
--
--     Given the name of a partioned table detach the cf partition and copy
--     the eligible rows from the temp_part# partition into the cf partition.
--
--     This version of the proc determines the detached partition rather than
--     accepting it as a parameter.
--      
--     Obviously a partition had better be detached before getting here.
--
--
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
--           
--
-- Usage:
--
--
-- History:    
--      2007/08/30   jicimi   created       
--      2007/11/05   jicimi   line 285 commented out 
--                              SET v_timestamp = v_timestamp - 1 MICROSECOND;
--      2007/11/22   jicimi   global temp table changes
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT (i_locktimeout_cnt)
--                            to pass to exec_sql_and_log and set_integrity procedure 
--      2012/04/25   leushe   changed the query to retrieve the CF partition new upper bound
--                            to check partitions where the status value is empty
--                            length(trim(status))=0
--                            Also added a loop to check to make sure the status is empty
--                            for all partitions, if not wait 30 seconds and check the status column
--                            for all partitions, if not wait 30 seconds again
--      2012/04/26   leushe   added new parameter i_detach_wait_sec to send to the sleep() call
--                            The value comes from the FMX_PARTITION_DEFINITION table.
--          
--         
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE v_table_part_name    VARCHAR(50);
  DECLARE v_partition          VARCHAR(50);
  DECLARE v_dynStmt            VARCHAR(1024);
  DECLARE v_lowerBoundStr      VARCHAR(30);
  DECLARE v_upperBoundStr      VARCHAR(30);
  DECLARE v_newUpperBoundStr   VARCHAR(30);
  DECLARE v_dayofweek          CHAR(1) ; 
  DECLARE v_newpartname        VARCHAR(50);     -- this is the name of the detached cf partition
  DECLARE v_cf_partition_name  VARCHAR(50);     -- this is the name of the detached cf table
  DECLARE v_workStr            VARCHAR(50);
  DECLARE v_workStr2           VARCHAR(50);
  DECLARE v_timestamp          TIMESTAMP;
  DECLARE v_date               DATE;
  DECLARE v_detached_partition VARCHAR(50);
  DECLARE o_detached_partition VARCHAR(50);
  DECLARE v_temp_table         VARCHAR(50);     -- the name of the gloabl temp table
  DECLARE v_invalid_part_cnt integer DEFAULT 0 ;


  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;

  DECLARE v_stmt STATEMENT;



  -- 
  -- get the name of the CF partition 
  --
  -- ******
  -- ******
  -- ******
  -- Should the i_cf_partition parameter be removed ?
  -- ******
  -- ******
  -- ******
  -- 
  SET v_partition = 
      ( select cast(datapartitionname as varchar(50))  
        from syscat.datapartitions a
        where tabname = i_tabname 
        and datapartitionname like '%CF%' 
        and tabschema = i_schema 
        and status != 'I'
      );


  -- 
  -- get the current min(lowerbound) of the 1st non CF partition 
  -- to use as the upperbound of the CF partition
  -- 
  -- 2007/09/16 - in preview mode, this creates a bug
  --              the oldest partition is never detached so the min(lowvalue) 
  --              value returned is actually from the partition that under ordinary
  --              cirsumstances would have been detached 
  --
  --              the following if compensates for that shortcomming
  --      
  SET v_upperBoundStr = 
      ( select cast( min(lowvalue) as varchar(30))
        from syscat.datapartitions a
        where tabname = i_tabname 
        and datapartitionname not like '%CF%'  
        and tabschema = i_schema 
        and length(trim(status))=0
      );

       
      
  IF ( i_preview_only_ind = '1' ) THEN
    SET v_upperBoundStr = 
        ( select cast( min(lowvalue) as varchar(30))
          from syscat.datapartitions a
          where tabname = i_tabname 
          and datapartitionname not like '%CF%'  
          and datapartitionname not like '%THE_END%'  
          and tabschema = i_schema 
          and status != 'I'
          and lowvalue > 
               ( select  min(lowvalue) 
                 from syscat.datapartitions b
                 where tabname = i_tabname 
                 and datapartitionname not like '%CF%'  
                 and datapartitionname not like '%THE_END%'  
                 and tabschema = i_schema 
                 and length(trim(status))=0 )
        );
  END IF;


  -- 
  -- get the current lowerbound  of the CF partition 
  -- 
  SET v_lowerBoundStr = 
      ( select cast(lowvalue as varchar(30))
        from syscat.datapartitions a
        where tabname = i_tabname 
        and datapartitionname like '%CF%' 
        and tabschema = i_schema 
        and status != 'I'
      );



  -- 
  -- get the name of the detached partition fcm_xxxx_temp_part#,  where # in 1..##
  --
  -- 2007/09/10 - in preview mode, this creates a bug 
  --              the oldest partition is never detached so the insert statement
  --              uses the name of the currently detached partition rather than the
  --              newly detached one 
  --
  --              resolved with the following if statement 
  --
  SET v_detached_partition =
      ( select cast(tabname as varchar(50))
        from syscat.tables a
        where ( tabname like i_tabname||'_DTCH_PART%' 
                and tabname not like i_tabname||'_DTCH_%CF%' ) 
        and tabschema = i_schema 
        and status != 'I'
      );
     

  IF ( i_preview_only_ind = '1' ) THEN
    SET v_detached_partition = 
        ( select i_tabname || '_DTCH_' || cast(datapartitionname as varchar(50))  
          from syscat.datapartitions a
          where ( tabname = i_tabname and datapartitionname not like '%CF%' 
                            and  datapartitionname not like '%THE_END%' 
                            and tabschema = i_schema 
                            and status != 'I' )
          and tabschema = i_schema 
          and lowvalue = ( select min(lowvalue) 
                           from syscat.datapartitions b
                           where a.tabschema = b.tabschema 
                           and a.tabname = b.tabname 
                           and b.datapartitionname not like '%CF%' 
                           and b.datapartitionname not like '%THE_END%' 
                           and tabschema = i_schema 
                           and status != 'I' )
        );
  END IF;   
  



  --
  --  get the dayofweek # to append to the cf partition name
  --  ( to allow the cf partition to be reattached )
  --
  SET v_dayOfWeek = ( select cast( dayofweek(current date) as char(1)) from sysibm.sysdummy1 ) ;


  --
  -- rename the cf partition to avoid the SQL0612N dup name, 42711 error
  --   always of the form PART_CF#
  --
  SET v_newpartname = substr(v_partition,1,7);
  SET v_newpartname = v_newpartname || v_dayOfWeek;
  
  --
  --
  --
  --  *******
  --  *******
  --  *******
  --  just for testing to avoid the SQL0612N dup name, SQLSTATE=42711 error
  --     append the current microsecond value onto the partition name 
  --  *******
  --  *******
  --  *******
  --
  SET v_newpartname = v_newpartname || '_' || 
      cast( microsecond(current timestamp) as char(6));
  --
  --  *******
  --  *******
  --  *******
  --  end test code
  --  *******
  --  *******
  --  *******
  --





  
  --
  -- the attach will fail if minvalue is used, so reset it if need be
  --
  IF ( v_lowerBoundStr = 'MINVALUE' ) THEN
     SET v_lowerBoundStr = '2006-01-01 00:00:00';
  END IF;


  
  --
  -- get the new upperBoundStr value
  --    take the str representation from the catalog    
  --    convert it to a date/timestamp
  --    then convert it back to a string for the attach call
  --

  SET v_upperBoundStr = STRIP(v_upperBoundStr, BOTH, '''');
  SET v_lowerBoundStr = STRIP(v_lowerBoundStr, BOTH, '''');

 

  --
  -- reformat the datapartitions.highvalue column into a format the timestamp_format
  --  function can sucessfully digest.
  -- 
  -- If the timestamp_format function only accepts one mask, WHY is the mask REQUIRED?!
  --
  SET v_workStr = 
       substr(v_upperBoundStr,1,4) || '-' || 
       substr(v_upperBoundStr,6,2) || '-' ||
       substr(v_upperBoundStr,9,2) || ' ' ||
       substr(v_upperBoundStr,12,2) || ':' || 
       substr(v_upperBoundStr,15,2) || ':' ||
       substr(v_upperBoundStr,18,2) ;


 
  SET v_timestamp = timestamp( v_workStr );
  -- 2007/11/05  commented out
  -- SET v_timestamp = v_timestamp - 1 MICROSECOND;
  SET v_newupperBoundStr = TO_CHAR(v_timestamp, 'YYYY-MM-DD HH24:MI:SS') ;

  


  -- 
  --  detach cf partition
  --
  CALL detach_partition(i_tabname, i_schema, v_partition, 
                        i_preview_only_ind, i_locktimeout_cnt, o_detached_partition, o_sqlcode );


  IF ( o_sqlcode != 0 ) THEN
    RETURN;
  END IF;
     

  --
  -- build the name of the newly detached cf table
  --
  SET v_cf_partition_name = o_detached_partition;

  
  --
  -- get the name of the global temp table
  --
  SET v_temp_table = i_tabname ||'_GTT' ;
  
  set v_invalid_part_cnt = ( select count(*)  
          from syscat.datapartitions 
          where  tabname = i_tabname 
          		and tabschema = i_schema 
            	and length(trim(status)) >0  
                  );

  WHILE (v_invalid_part_cnt > 0 ) DO

        call sleep(i_detach_wait_sec);
  		set v_invalid_part_cnt = ( select count(*)  
          from syscat.datapartitions 
          where  tabname = i_tabname 
          		and tabschema = i_schema 
            	and length(trim(status)) >0 
                  );
  END WHILE;
  --
  -- insert eligible rows from DTCH_PART# => ( v_detached_partition )  
  --      into DTCH_CF# => ( i_cf_partition )
  --
  CALL copy_cf_rows (i_tabname, i_schema, v_temp_table,
                     v_cf_partition_name, i_preview_only_ind,i_locktimeout_cnt, /*10000,*/ o_sqlcode );

  IF ( o_sqlcode != 0 ) THEN
    RETURN;
  END IF;
     

  --
  -- attach cf partition with new upper BoundStr
  --
  CALL attach_partition(i_tabname, i_schema, v_lowerBoundStr, v_newupperBoundStr, 
                        v_partition, v_newpartname, i_preview_only_ind,i_locktimeout_cnt,i_partition_datatype, o_sqlcode );

  IF ( o_sqlcode != 0 ) THEN
    RETURN;
  END IF;
     


  --
  -- set the ri back on
  --
  CALL set_integrity_on ( i_tabname, i_schema, i_preview_only_ind,i_locktimeout_cnt, o_sqlcode );


END P1
@


CREATE PROCEDURE roll_partition   ( IN i_tabname VARCHAR(50),
                                    IN i_schema  VARCHAR(30)
                                    )
SPECIFIC ROLL_PARTITION
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  ROLL_PARTITION
--
--
-- Description:
--     Top level entry point for the partitioning maintenance.
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
-- 
--
-- Usage:
--
--
-- History:    
--      2008/07/14   leushe   created. Added parameter to check run the Sanity Check or not    
--         					  '0'-- Perform Sanity Check Call
--         					  '1'-- Do not perform Sanity Check Call
--	                          Added parameter to keep confirmed frauds or not    
--         					  '1'-- keep confirmed frauds 
--         					  '0'-- Do not keep confirmed frauds
--      2009/08/24   leushe   Changed the proc to not return explicit parameters, but return 
--                            the sqlcode via a result set. this changed the signature of the proc
--                            This change was made so SAS could call the proc, as it doesn't handle explicit
--                            Output parameters
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT Column on the partition_definition table    
--      2012/04/26   leushe   Added support for the DETACH_WAIT_SEC Column on the partition_definition table, passed to call
--                            to the update_cf_partitions proc
--
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE v_rowCount        INTEGER   DEFAULT 0;
  DECLARE v_detached_table  VARCHAR(50);

  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;
  DECLARE v_sqlcode integer DEFAULT 0;
  DECLARE v_sqlstate varchar(50) DEFAULT '00000' ;
  DECLARE v_sqlmsg varchar(30000)  ;
  DECLARE l_preview_only_ind CHAR(1) DEFAULT '0';
  DECLARE l_chk_sanity_ind CHAR(1) DEFAULT '0';
  DECLARE l_keep_fraud_ind CHAR(1) DEFAULT '0';
  DECLARE l_partition_datatype CHAR(9) DEFAULT '0';
  DECLARE l_locktimeout_cnt INTEGER DEFAULT 3;
  DECLARE l_detach_wait_sec integer DEFAULT 30;
    
      /* Declare cursor to return result set to calling program */ 
        
        DECLARE c_result CURSOR WITH RETURN FOR
            SELECT   v_sqlcode as OUT_SQLCODE
                    ,v_sqlstate as OUT_SQLSTATE
            FROM    SYSIBM.SYSDUMMY1;
  
      /* Initialize variables */
        VALUES (SQLSTATE, SQLCODE) INTO v_sqlstate, v_sqlcode;
        
        
  -- Z.  upcase all arguments 
  SET i_tabname = upper(i_tabname);
  SET i_schema = upper(i_schema);
  SET v_sqlstate = i_tabname ;
  
  --
  -- verify i_tabname is indeed a partitioned table
  --
  SET v_rowCount = ( select count(*) from fmx_partition_definition where table_name = i_tabname );

  IF ( v_rowCount != 1 ) THEN
    SET v_sqlcode = -1;
    SET v_sqlstate = '-1';
    SET v_sqlmsg = '';
    CALL audit_partition_action(i_tabname, i_schema, NULL, i_tabname||' is not partition eligible.',
                                '0', v_sqlcode, v_sqlstate, v_sqlmsg);
    SET v_sqlstate = i_tabname;                                
    OPEN c_result;

    RETURN -1;
  END IF;

  SELECT LOCKTIMEOUT_RETRY_COUNT,PREVIEW_ONLY_IND,SKIP_SANITY_CHECK_IND,KEEP_FRAUD_IND,PARTITION_DATATYPE,DETACH_WAIT_SEC 
  		INTO l_locktimeout_cnt,L_PREVIEW_ONLY_IND,L_CHK_SANITY_IND,L_KEEP_FRAUD_IND,L_PARTITION_DATATYPE,L_DETACH_WAIT_SEC
  FROM FMX_PARTITION_DEFINITION
  WHERE TABLE_NAME = I_TABNAME;



  IF ( l_chk_sanity_ind = '0' ) THEN

    -- 0. sanity check its ok to go on
    call sanity_check_partition(i_tabname, i_schema, l_preview_only_ind, l_keep_fraud_ind, v_sqlcode) ;

    IF ( v_sqlcode != 0 ) THEN    
      OPEN c_result;

      RETURN -1;
    END IF; 
  END IF; 

  call attach_partition(i_tabname, i_schema, l_preview_only_ind, l_locktimeout_cnt, l_partition_datatype,  v_sqlcode);   
  IF ( v_sqlcode != 0 ) THEN
      OPEN c_result;

      RETURN -1;

  END IF;

  
  IF ( l_keep_fraud_ind = '1' ) THEN
  --
  -- 1b. create a global temp table and copy eligible rows into from the base table
    call create_partition_temp_table(i_tabname, i_schema, l_preview_only_ind, l_locktimeout_cnt, v_detached_table, v_sqlcode);

    IF ( v_sqlcode != 0 ) THEN
      OPEN c_result;

      RETURN -1;
    END IF;
  END IF; 
   
  -- 2. detach oldest
  call detach_partition(i_tabname, i_schema, l_preview_only_ind, l_locktimeout_cnt, v_detached_table, v_sqlcode);

  IF ( v_sqlcode != 0 ) THEN
      OPEN c_result;

      RETURN -1;
  END IF;


 
  IF ( l_keep_fraud_ind = '1' ) THEN
  -- 3. update_cf
    call update_cf_partition(i_tabname, i_schema, 'PART_CF', l_preview_only_ind,l_locktimeout_cnt, l_partition_datatype,l_detach_wait_sec, v_sqlcode);
   
    IF ( v_sqlcode != 0 ) THEN
      OPEN c_result;

      RETURN -1;
    END IF;
  END IF; 

   
  -- 4. empty oldest
  call truncate(v_detached_table, i_schema, l_preview_only_ind, l_locktimeout_cnt, v_sqlcode);
    IF ( v_sqlcode != 0 ) THEN
      OPEN c_result;

      RETURN -1;
    END IF;


  -- 5. insert current timestamp into definition table
  --    this value is used to ensure the code does not
  --    execute more frequently than it is supposed to
  IF ( l_preview_only_ind = '0' ) THEN
      update fmx_partition_definition 
         set last_run_timestamp = ( current timestamp )
         where table_name = i_tabname ;

  END IF;
  
  VALUES ( SQLCODE) INTO  v_sqlcode;
  COMMIT;
  OPEN c_result;
   
END P1
@




