call sysproc.set_routine_opts('ISOLATION UR') @

drop specific procedure truncate @
drop specific procedure roll_partition    @


CREATE PROCEDURE truncate         ( IN i_tabname VARCHAR(50),
                                    IN i_schema  VARCHAR(30),
                                    IN i_preview_only_ind CHAR(1),
                                    IN i_locktimeout_cnt INTEGER,
                                    IN i_detach_wait_sec INTEGER,
                                    OUT o_sqlcode INT  )
SPECIFIC truncate         
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  truncate
--
--
-- Description:
--     Given the name of a table truncate it by invoking the loader.
--
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
-- 
--
-- Usage:
--
--
-- History:    
--      2007/09/06   jicimi   created       
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT 
--                            to pass to exec_sql_and_log procedure 
--      2012/04/26   leushe   added new parameter i_detach_wait_sec to send to the sleep() call
--                            The value comes from the FMX_PARTITION_DEFINITION table.
--            
--           
--          
--         
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE v_table_part_name VARCHAR(50);
  DECLARE v_partition       VARCHAR(50);
  DECLARE v_dynStmt         VARCHAR(1024);
  DECLARE v_dynStmtParm     VARCHAR(1024);
  DECLARE v_invalid_part_cnt integer DEFAULT 0 ;

  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;
  DECLARE v_sqlcode integer;
  DECLARE v_sqlstate char(5) ;
  
  DECLARE v_stmt STATEMENT;


  --
  -- build the dynamic truncate statement via import /dev/null
  --
  SET v_dynStmtParm = 'IMPORT FROM /dev/null of del replace into '||i_schema||'.'||i_tabname;

  --
  -- create the dynamic sql statement
  -- 
  SET v_dynStmt = 'call sysproc.admin_cmd (?)';
  
  set v_invalid_part_cnt = ( select count(*)  
          from syscat.datapartitions 
          where  tabname = i_tabname 
          		and tabschema = i_schema 
            	and length(trim(status)) >0  
                  );

  WHILE (v_invalid_part_cnt > 0 ) DO

        call sleep(i_detach_wait_sec);
  		set v_invalid_part_cnt = ( select count(*)  
          from syscat.datapartitions 
          where  tabname = i_tabname 
          		and tabschema = i_schema 
            	and length(trim(status)) >0 
                  );
  END WHILE;

  CALL exec_sql_and_log 
        (i_tabname, i_schema, v_partition, i_preview_only_ind, i_locktimeout_cnt,v_dynStmt, v_dynStmtParm, o_sqlcode) ;


END P1
@



            @
CREATE PROCEDURE roll_partition   ( IN i_tabname VARCHAR(50),
                                    IN i_schema  VARCHAR(30)
                                    )
SPECIFIC ROLL_PARTITION
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  ROLL_PARTITION
--
--
-- Description:
--     Top level entry point for the partitioning maintenance.
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
-- 
--
-- Usage:
--
--
-- History:    
--      2008/07/14   leushe   created. Added parameter to check run the Sanity Check or not    
--         					  '0'-- Perform Sanity Check Call
--         					  '1'-- Do not perform Sanity Check Call
--	                          Added parameter to keep confirmed frauds or not    
--         					  '1'-- keep confirmed frauds 
--         					  '0'-- Do not keep confirmed frauds
--      2009/08/24   leushe   Changed the proc to not return explicit parameters, but return 
--                            the sqlcode via a result set. this changed the signature of the proc
--                            This change was made so SAS could call the proc, as it doesn't handle explicit
--                            Output parameters
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT Column on the partition_definition table    
--      2012/04/26   leushe   Added support for the DETACH_WAIT_SEC Column on the partition_definition table, passed to call
--                            to the update_cf_partitions proc
--
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE v_rowCount        INTEGER   DEFAULT 0;
  DECLARE v_detached_table  VARCHAR(50);

  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;
  DECLARE v_sqlcode integer DEFAULT 0;
  DECLARE v_sqlstate varchar(50) DEFAULT '00000' ;
  DECLARE v_sqlmsg varchar(30000)  ;
  DECLARE l_preview_only_ind CHAR(1) DEFAULT '0';
  DECLARE l_chk_sanity_ind CHAR(1) DEFAULT '0';
  DECLARE l_keep_fraud_ind CHAR(1) DEFAULT '0';
  DECLARE l_partition_datatype CHAR(9) DEFAULT '0';
  DECLARE l_locktimeout_cnt INTEGER DEFAULT 3;
  DECLARE l_detach_wait_sec integer DEFAULT 30;
    
      /* Declare cursor to return result set to calling program */ 
        
        DECLARE c_result CURSOR WITH RETURN FOR
            SELECT   v_sqlcode as OUT_SQLCODE
                    ,v_sqlstate as OUT_SQLSTATE
            FROM    SYSIBM.SYSDUMMY1;
  
      /* Initialize variables */
        VALUES (SQLSTATE, SQLCODE) INTO v_sqlstate, v_sqlcode;
        
        
  -- Z.  upcase all arguments 
  SET i_tabname = upper(i_tabname);
  SET i_schema = upper(i_schema);
  SET v_sqlstate = i_tabname ;
  
  --
  -- verify i_tabname is indeed a partitioned table
  --
  SET v_rowCount = ( select count(*) from fmx_partition_definition where table_name = i_tabname );

  IF ( v_rowCount != 1 ) THEN
    SET v_sqlcode = -1;
    SET v_sqlstate = '-1';
    SET v_sqlmsg = '';
    CALL audit_partition_action(i_tabname, i_schema, NULL, i_tabname||' is not partition eligible.',
                                '0', v_sqlcode, v_sqlstate, v_sqlmsg);
    SET v_sqlstate = i_tabname;                                
    OPEN c_result;

    RETURN -1;
  END IF;

  SELECT LOCKTIMEOUT_RETRY_COUNT,PREVIEW_ONLY_IND,SKIP_SANITY_CHECK_IND,KEEP_FRAUD_IND,PARTITION_DATATYPE,DETACH_WAIT_SEC 
  		INTO l_locktimeout_cnt,L_PREVIEW_ONLY_IND,L_CHK_SANITY_IND,L_KEEP_FRAUD_IND,L_PARTITION_DATATYPE,L_DETACH_WAIT_SEC
  FROM FMX_PARTITION_DEFINITION
  WHERE TABLE_NAME = I_TABNAME;



  IF ( l_chk_sanity_ind = '0' ) THEN

    -- 0. sanity check its ok to go on
    call sanity_check_partition(i_tabname, i_schema, l_preview_only_ind, l_keep_fraud_ind, v_sqlcode) ;

    IF ( v_sqlcode != 0 ) THEN    
      OPEN c_result;

      RETURN -1;
    END IF; 
  END IF; 

  call attach_partition(i_tabname, i_schema, l_preview_only_ind, l_locktimeout_cnt, l_partition_datatype,  v_sqlcode);   
  IF ( v_sqlcode != 0 ) THEN
      OPEN c_result;

      RETURN -1;

  END IF;

  
  IF ( l_keep_fraud_ind = '1' ) THEN
  --
  -- 1b. create a global temp table and copy eligible rows into from the base table
    call create_partition_temp_table(i_tabname, i_schema, l_preview_only_ind, l_locktimeout_cnt, v_detached_table, v_sqlcode);

    IF ( v_sqlcode != 0 ) THEN
      OPEN c_result;

      RETURN -1;
    END IF;
  END IF; 
   
  -- 2. detach oldest
  call detach_partition(i_tabname, i_schema, l_preview_only_ind, l_locktimeout_cnt, v_detached_table, v_sqlcode);

  IF ( v_sqlcode != 0 ) THEN
      OPEN c_result;

      RETURN -1;
  END IF;


 
  IF ( l_keep_fraud_ind = '1' ) THEN
  -- 3. update_cf
    call update_cf_partition(i_tabname, i_schema, 'PART_CF', l_preview_only_ind,l_locktimeout_cnt, l_partition_datatype,l_detach_wait_sec, v_sqlcode);
   
    IF ( v_sqlcode != 0 ) THEN
      OPEN c_result;

      RETURN -1;
    END IF;
  END IF; 

   
  -- 4. empty oldest
  call truncate(v_detached_table, i_schema, l_preview_only_ind, l_locktimeout_cnt, l_detach_wait_sec, v_sqlcode);
    IF ( v_sqlcode != 0 ) THEN
      OPEN c_result;

      RETURN -1;
    END IF;


  -- 5. insert current timestamp into definition table
  --    this value is used to ensure the code does not
  --    execute more frequently than it is supposed to
  IF ( l_preview_only_ind = '0' ) THEN
      update fmx_partition_definition 
         set last_run_timestamp = ( current timestamp )
         where table_name = i_tabname ;

  END IF;
  
  VALUES ( SQLCODE) INTO  v_sqlcode;
  COMMIT;
  OPEN c_result;
   
END P1
@








